# oppgave1.tcl
#
# Topology:
# h3 ---\                   /--- h7
#        \                 /
# h4 ----- n1 --- n0 ---- n2 ----- h8
#        /(edge) (core)    \
# h5 ---/                   \--- h9
#      /                     \
# h6--/                       \--h10
#
# (1) TCP transmission pairs: h3-h7, h4-h8, h6-h10:
#     tcp/bulk transfer
#     
# (2) UDP transmission pairs: h5-h9:
#     CBR at 1.2e5 bps
#
# (3) No QOS configured


set bandwidth 1.0e6; # bps
set propDelay 0.05;   # second
set port 1001; # for CBR traffic

set toSetupCBR 1
set toSetupHostTC 0
set toSetupEdgeTC 0
set toSetupCoreQ 0
set toSetupPlot 1
set toSetupPlotFile 1
set toSetupInspect 1

cd [mkdir -q drcl.comp.Component /oppgave_noQOS]

# create the topology
puts "create topology..."
set link_ [java::new drcl.inet.Link]
$link_ setPropDelay $propDelay; # 50ms
set adjMatrix_ [java::new {int[][]} 11 {{1 2} {0 3 4 5 6} {0 7 8 9 10} {1} {1} {1} {1} {2} {2} {2} {2}}]
java::call drcl.inet.InetUtil createTopology [! .] $adjMatrix_ $link_

puts "create builders..."

# CSLBuilder:
set ccb [mkdir drcl.inet.core.CSLBuilder .cslBuilder]
set ecb [cp $ccb .edgeRouterCSLBuilder]
set hcb [cp $ccb .hostCSLBuilder]

if $toSetupCoreQ {
	# set up CSL builder for core router
	mkdir drcl.diffserv.scheduling.pq $ccb/q1; # only at interface 1
}
if $toSetupEdgeTC {
	# set up CSL builder for edge router
	mkdir drcl.diffserv.TrafficConditioner $ecb/pf0_0; # only at interface 0
}
if $toSetupHostTC {
	# set up CSL builder for source host (containing marker)
	mkdir drcl.diffserv.TrafficConditioner $hcb/pf0_0;
}

# regular node builder:
set nb [mkdir drcl.inet.NodeBuilder .routerBuilder]
$nb setBandwidth 10.0e6; # 10Mbps

puts "build..."
$nb build [! n0] $ccb
$nb build [! n1] $ecb
$nb build [! n2,h9]
$nb build [! h5] $hcb
$nb build [! h3,h4,h6] $hcb {
	tcp 				drcl.inet.transport.TCP
	source 	-/tcp 		drcl.inet.application.BulkSource
}
$nb build [! h7,h8,h10] {
	tcpsink 			drcl.inet.transport.TCPSink
	sink 	-/tcpsink 	drcl.inet.application.BulkSink
}
! h3,h4,h6,h7,h9,h10/tcp* setMSS 512; #byte
! h3,h4,h6,h7,h9,h10/s* setDataUnit 512

#set cbr source
if $toSetupCBR {
# Arguments: pkt_size interval, rate 1.2e05 bps
	set traffic [java::new drcl.net.traffic.traffic_PacketTrain 150 0.01]; 
	java::call drcl.inet.InetUtil createTrafficSource $traffic "source" [! h5] [! h9] 0 $port
}

# Configure the bottleneck bandwidth and buffer size
! n0 setBandwidth 1 $bandwidth
! n0 setBuffer 1 20 packet; # ~10 packets at interface 0


# setup TCP parameters
! h3/tcp setPeer 7
! h4/tcp setPeer 8
! h6/tcp setPeer 10

puts "setup static routes..."
java::call drcl.inet.InetUtil setupRoutes [! h3] [! h7] "bidirection"
java::call drcl.inet.InetUtil setupRoutes [! h4] [! h8] "bidirection"
java::call drcl.inet.InetUtil setupRoutes [! h5] [! h9] "bidirection"
java::call drcl.inet.InetUtil setupRoutes [! h6] [! h10] "bidirection"

if {$toSetupPlot | $toSetupPlotFile} {
puts "Set up TrafficMonitor & Plotter..."
set plot_ [mkdir drcl.comp.tool.Plotter .plot]
if {!$toSetupPlot} {
	set fileName_ "oppgave_noQOS.plot"
	puts "Set up file '$fileName_' to store results..."
	setflag plot false $plot_
	set file_ [mkdir drcl.comp.io.FileComponent .file]
	$file_ open $fileName_
	connect -c $plot_/.output@ -to $file_/in@
}
foreach i {7 8 10} {
	set j [expr $i-7]
	set k [expr $i-4]
	set tm$k\_ [mkdir drcl.net.tool.TrafficMonitor .tm$k]
	connect -c h$i/csl/6@up -to .tm$k/in@
	connect -c .tm$k/bytecount@ -to $plot_/$j@0
	connect -c h$k/tcp/cwnd@ -to $plot_/$j@1

}
foreach i {9} {
	set j [expr $i-7]
	set k [expr $i-4]
	set tm$k\_ [mkdir drcl.net.tool.TrafficMonitor .tm$k]
	connect -c h$i/csl/$port@up -to .tm$k/in@
	connect -c .tm$k/bytecount@ -to $plot_/$j@0
}
#foreach i {5} {
#	set j [expr $i]
#	set k [expr $i-4]
#	set tm$k\_ [mkdir drcl.net.tool.TrafficMonitor .tm$k]
#	connect -c h$i/csl/$port@up -to .tm$k/in@
#	connect -c .tm$k/bytecount@ -to $plot_/$j@0
#}
! .tm? configure 4.0 0.5; # window size, update interval
}


setflag garbagedisplay true recursively n0/csl/

puts "simulation begins..."	
set sim [attach_simulator event .]

$sim stop

if $toSetupCBR {
puts "start cbr source"
run h5 
}

puts "start tcp sources"
#! h3-4/tcp setImplementation vegas
run h3,h4
run h6 -at 50.0 

script {puts [$sim getTime]} -period 10.0 -on $sim
$sim resumeTo 200
#script {puts [$sim diag]; quit} -at 150.0 -on $sim
#$sim resume

puts done!
